from hbshare.quant.load_data import load_funds_data, load_funds_outperformance
import pandas as pd
import numpy as np
import pyecharts.options as opts
from datetime import datetime, timedelta
from pyecharts.charts import Line, Tab, Grid
import pymysql
from sqlalchemy import create_engine
from hbshare.quant.cons import sql_write_path_hb



def nav_lines(fund_list, start_date, end_date, title='', zz=False):
    pymysql.install_as_MySQLdb()

    db_path = sql_write_path_hb['work']
    cal_path = sql_write_path_hb['daily']

    if zz:
        funds_data = load_funds_outperformance(
            fund_list=fund_list,
            first_date=start_date,
            end_date=end_date,
            db_path=db_path,
            cal_db_path=cal_path
        )['eav']
    else:
        funds_data = load_funds_data(
            fund_list=fund_list,
            first_date=start_date,
            end_date=end_date,
            db_path=db_path,
            cal_db_path=cal_path,
            # freq='',
            # fillna=False
        )

    web = Line(
        init_opts=opts.InitOpts(
            page_title=title,
            width='700px',
            height='500px',
            # theme=ThemeType.DARK
        )
    ).set_global_opts(
        tooltip_opts=opts.TooltipOpts(is_show=True),
        toolbox_opts=opts.ToolboxOpts(is_show=True),
        xaxis_opts=opts.AxisOpts(type_="category"),
        yaxis_opts=opts.AxisOpts(
            type_="value",
            axistick_opts=opts.AxisTickOpts(is_show=True),
            splitline_opts=opts.SplitLineOpts(is_show=True),
        ),
    ).add_xaxis(
        xaxis_data=funds_data['t_date'].tolist()
    )
    funds = funds_data.columns.tolist()

    # fund_monthly = ['德劭锐哲中国']
    # for m in fund_monthly:
    #     if m in funds:
    #         fund_data = load_funds_data(
    #             fund_list=pd.read_sql_query(
    #                 'select * from fund_list where `name`="' + m + '"', engine
    #             ),
    #             first_date=start_date,
    #             end_date=end_date,
    #             freq='month'
    #         )
    #         funds_data[m] = fund_data[m]
    funds.remove('t_date')
    for j in funds:
        nav_data = funds_data[j] / funds_data[funds_data[j] > 0][j].tolist()[0]
        web.add_yaxis(
            series_name=j,
            y_axis=nav_data.round(4).tolist(),
            symbol="emptyCircle",
            is_symbol_show=True,
            label_opts=opts.LabelOpts(is_show=False),
        )

    web.set_global_opts(
        xaxis_opts=opts.AxisOpts(
            is_scale=True,
            type_="category", boundary_gap=False
        ),
        yaxis_opts=opts.AxisOpts(
            is_scale=True,
            splitarea_opts=opts.SplitAreaOpts(
                is_show=True, areastyle_opts=opts.AreaStyleOpts(opacity=1)
            ),
        ),
        legend_opts=opts.LegendOpts(
            # type_='scroll',
            pos_top='5%'
        ),
        datazoom_opts=[
            opts.DataZoomOpts(range_start=0, range_end=100),
            # opts.DataZoomOpts(pos_left="5%", xaxis_index=0),
            # opts.DataZoomOpts(pos_right="5%", xaxis_index=1),
            opts.DataZoomOpts(type_="inside")
        ],
        title_opts=opts.TitleOpts(title=title),
        tooltip_opts=opts.TooltipOpts(trigger="axis")
    )

    return web


def gen_grid(end_date, class0, type0, zz):
    pymysql.install_as_MySQLdb()

    db_path = sql_write_path_hb['work']
    cal_path = sql_write_path_hb['daily']

    engine = create_engine(sql_write_path_hb['work'])
    start_date = end_date - timedelta(days=365 * 2 + 7)
    start_date2 = end_date - timedelta(days=365 + 7)

    fund_list = pd.read_sql_query(
        'select * from fund_list where `class`="' + class0
        + '" and `type`="' + type0
        + '" and `stopped`!= 1 order by `name`', engine
    )

    grid_nav = Grid(init_opts=opts.InitOpts(width="1200px", height="500px"))

    web = nav_lines(
        fund_list=fund_list,
        start_date=start_date,
        end_date=end_date,
        zz=zz
    )

    web2 = nav_lines(
        fund_list=fund_list,
        start_date=start_date2,
        end_date=end_date,
        zz=zz
    )

    grid_nav.add(
        web.set_global_opts(
            title_opts=opts.TitleOpts(
                title=class0 + ' ' + type0 + ' 近两年: '
                      + start_date.strftime('%Y/%m/%d') + '~' + end_date.strftime('%Y/%m/%d')
                ,
                pos_right="5%"
            ),
            tooltip_opts=opts.TooltipOpts(trigger="axis"),
            legend_opts=opts.LegendOpts(
                pos_top="8%"
            )
        ),
        grid_opts=opts.GridOpts(pos_left='55%', pos_top="18%")
    ).add(
        web2.set_global_opts(
            title_opts=opts.TitleOpts(
                title=class0 + ' ' + type0 + ' 近一年: '
                      + start_date2.strftime('%Y/%m/%d') + '~' + end_date.strftime('%Y/%m/%d')
                ,
                pos_left="5%"
            ),
            tooltip_opts=opts.TooltipOpts(trigger="axis"),
            legend_opts=opts.LegendOpts(
                pos_top="8%"
            )
        ),
        grid_opts=opts.GridOpts(pos_right='55%', pos_top="18%")
    )
    return grid_nav


def gen_tab(end_date, class0):
    pymysql.install_as_MySQLdb()

    db_path = sql_write_path_hb['work']
    cal_path = sql_write_path_hb['daily']

    engine = create_engine(sql_write_path_hb['work'])
    start_date = end_date - timedelta(days=365 * 2 + 7)
    start_date2 = end_date - timedelta(days=365 + 7)

    fund_list = pd.read_sql_query(
        'select * from fund_list where `class`="' + class0 + '" and `stopped`!= 1 order by `name`', engine
    )
    class_list = fund_list['type'].drop_duplicates().tolist()
    class_dict = {
        class0: class_list,
        # '中性': class_list_zx,
        # '指增': class_list_zz
    }
    tab = Tab(
        page_title=class0 + '量化产品净值' + end_date.strftime('%Y%m%d'),
    )

    for c in class_dict:
        class_list = class_dict[c]
        if c == '指增':
            zz = True
        else:
            zz = False

        for i in range(len(class_list)):
            grid_nav = Grid(init_opts=opts.InitOpts(width="1200px", height="500px"))

            web = nav_lines(
                fund_list=fund_list[
                    np.array(fund_list['type'] == class_list[i]) & np.array(fund_list['class'] == c)
                    ].reset_index(drop=True),
                start_date=start_date,
                end_date=end_date,
                zz=zz
            )

            web2 = nav_lines(
                fund_list=fund_list[
                    np.array(fund_list['type'] == class_list[i]) & np.array(fund_list['class'] == c)
                    ].reset_index(drop=True),
                start_date=start_date2,
                end_date=end_date,
                zz=zz
            )

            grid_nav.add(
                web.set_global_opts(
                    title_opts=opts.TitleOpts(
                        title=class_list[i] + ' 近两年 '
                        + start_date.strftime('%Y/%m/%d') + '~' + end_date.strftime('%Y/%m/%d')
                        ,
                        pos_right="5%"
                    ),
                    tooltip_opts=opts.TooltipOpts(trigger="axis"),
                    legend_opts=opts.LegendOpts(
                        pos_top="8%",
                        # pos_right="20%",
                        # pos_left="20%"
                    )
                ),
                grid_opts=opts.GridOpts(pos_left='55%', pos_top="18%")
            ).add(
                web2.set_global_opts(
                    title_opts=opts.TitleOpts(
                        title=class_list[i] + ' 近一年 '
                        + start_date2.strftime('%Y/%m/%d') + '~' + end_date.strftime('%Y/%m/%d')
                        ,
                        pos_left="5%"
                    ),
                    tooltip_opts=opts.TooltipOpts(trigger="axis"),
                    legend_opts=opts.LegendOpts(
                        pos_top="8%",
                        # pos_right="20%",
                        # pos_left="20%"
                    )
                ),
                grid_opts=opts.GridOpts(pos_right='55%', pos_top="18%")
            )
            name = c + ':' + class_list[i]
            if c == '指增':
                name += '（超额）'
            tab.add(grid_nav, name)
    return tab





#!/usr/bin/python
#coding:utf-8

"""
@author: Meng.lv
@contact: meng.lv@howbuy.com
@software: PyCharm
@file: __init__.py.py
@time: 2020/6/15 10:04
"""

from hbshare.fund.nav import (get_fund_newest_nav_by_code)

from hbshare.simu.corp import (get_simu_corp_list_by_keyword, get_prod_list_by_corp_code)
from hbshare.simu.nav import (get_simu_nav_by_code)

from hbshare.base.data_pro import (hb_api)
from hbshare.base.upass import (get_token, set_token)

from hbshare.quant.load_data import (load_calendar_extra, load_funds_data, load_funds_outperformance)
from hbshare.quant.gen_charts import (gen_tab, nav_lines)






# -*- coding:utf-8 -*-
"""
Created on 2020/6/15
@author: Meng Lv
@group: Howbuy FDC
@contact: meng.lv@howbuy.com
"""
VERSION = '1.0.1'
P_TYPE = {'http': 'http://', 'ftp': 'ftp://', 'https': 'https://'}
FORMAT = lambda x: '%.2f' % x
FORMAT4 = lambda x: '%.4f' % x
DOMAINS = {'hbcgi': 'data.howbuy.com', 's': 's.howbuy.com'}

##########################################################################
# 基金数据列名
HOWBUY_SIMU_CORP = ['tsname', 'tsshortName', 'tscode']
HOWBUY_SIMU_PROD_LIST = ['fundCode', 'fundName', 'hb1n','jjjz', 'jzrq', 'nhbdl', 'nhsyl']
HOWBUY_SIMU_NAV = ['jzrq', 'jjjz', 'ljjz','fqdwjz', 'jjjzStr', 'ljjzStr']
#test
##########################################################################
# 数据源URL
HOWBUY_SIMU_CORP_SEARCH = "%s%s/simu_searchdata_new.do?q=%s&fq=600&type=1"
HOWBUY_SIMU_GET_PROD_LIST_BY_CORP_CODE = "%s%s/cgi/simu/v634/company.json?jgdm=%s"
HOWBUY_SIMU_GET_NAV_LIST_BY_CODE = "%s%s/cgi/fund/historyfundnetvalueofpage.json?fundCode=%s&isPrivate=1&startDate=%s&endDate=%s&pageCount=%s&pageNum=%s"

##########################################################################
DATA_GETTING_TIPS = '[Getting data:]'
DATA_GETTING_FLAG = '#'
DATA_ROWS_TIPS = '%s rows data found.Please wait for a moment.'
DATA_INPUT_ERROR_MSG = 'date input error.'
NETWORK_URL_ERROR_MSG = '获取失败，请检查网络和URL'
DATE_CHK_MSG = '年度输入错误：请输入1989年以后的年份数字，格式：YYYY'
DATE_CHK_Q_MSG = '季度输入错误：请输入1、2、3或4数字'
TOP_PARAS_MSG = 'top有误，请输入整数或all.'
LHB_MSG = '周期输入有误，请输入数字5、10、30或60'

OFT_MSG = u'开放型基金类型输入有误，请输入all、equity、mix、bond、monetary、qdii'

DICT_NAV_EQUITY = {
    'fbrq': 'date',
    'jjjz': 'value',
    'ljjz': 'total',
    'change': 'change'
}

DICT_NAV_MONETARY = {
    'fbrq': 'date',
    'nhsyl': 'value',
    'dwsy': 'total',
    'change': 'change'
}

import sys
PY3 = (sys.version_info[0] >= 3)


def _write_head():
    sys.stdout.write(DATA_GETTING_TIPS)
    sys.stdout.flush()


def _write_console():
    sys.stdout.write(DATA_GETTING_FLAG)
    sys.stdout.flush()


def _write_tips(tip):
    sys.stdout.write(DATA_ROWS_TIPS % tip)
    sys.stdout.flush()


def _write_msg(msg):
    sys.stdout.write(msg)
    sys.stdout.flush()


def _check_input(year, quarter):
    if isinstance(year, str) or year < 1989:
        raise TypeError(DATE_CHK_MSG)
    elif quarter is None or isinstance(quarter, str) or quarter not in [1, 2, 3, 4]:
        raise TypeError(DATE_CHK_Q_MSG)
    else:
        return True



# -*- coding:utf-8 -*-

"""
获取基金净值数据接口
Created on 2020/6/15
@author: Meng Lv
@group: Howbuy FDC
@contact: meng.lv@howbuy.com
"""

from __future__ import division
import time
import pandas as pd
from hbshare.fund import cons as ct
import hbshare as hbs


def get_fund_newest_nav_by_code(code, retry_count=3, pause=0.01, timeout=10):
    """
        获得公募基金最新净值（含最新回报，区间回报）
    :param code:
    :return:
    """
    api = hbs.hb_api()
    for _ in range(retry_count):
        time.sleep(pause)
        ct._write_console()
        url = ct.HOWBUY_FUND_NEWEST_NAV % (ct.P_TYPE['https'], ct.DOMAINS['hbcgi'], code)

        # request = Request(ct.HOWBUY_FUND_NEWEST_NAV %
        #                   (ct.P_TYPE['https'], ct.DOMAINS['hbcgi'],
        #                    code))
        # text = urlopen(request, timeout=timeout).read()
        # text = text.decode('gbk')

        org_js = api.query(url)
        status_code = int(org_js['common']['responseCode'])
        if status_code != 1:
            status = str(org_js['common']['responseContent'])
            raise ValueError(status)

        if 'opens' not in org_js:
            status = "未查询到该基金数据"
            raise ValueError(status)

        data = org_js['opens']
        fund_df = pd.DataFrame(data, columns=ct.HOWBUY_FUND_NEWEST_NAV_COLUMNS)
        fund_df['jjjz'] = fund_df['jjjz'].astype(float)
        fund_df['ljjz'] = fund_df['ljjz'].astype(float)
        fund_df['hbdr'] = fund_df['hbdr'].astype(float)
        fund_df['hb1y'] = fund_df['hb1y'].astype(float)
        fund_df['hb3y'] = fund_df['hb3y'].astype(float)
        fund_df['hb6y'] = fund_df['hb6y'].astype(float)
        fund_df['hbjn'] = fund_df['hbjn'].astype(float)
        fund_df['hb1n'] = fund_df['hb1n'].astype(float)
        fund_df['zfxz'] = fund_df['zfxz'].astype(float)
        return fund_df
    raise IOError(ct.NETWORK_URL_ERROR_MSG)



#!/usr/bin/python
#coding:utf-8

"""
@author: Meng.lv
@contact: meng.lv@howbuy.com
@software: PyCharm
@file: fund_nav_test.py
@time: 2020/6/15 10:50
"""

import hbshare as hbs


def test_get_fund_newest_nav():
    hbs.set_token("qwertyuisdfghjkxcvbn1000")
    data = hbs.get_fund_newest_nav_by_code('000004')
    print(data)

def get_simu_corp_list_by_keyword():
    hbs.set_token("qwertyuisdfghjkxcvbn1000")
    #data = hbs.get_simu_corp_list_by_keyword('黑石')
    data = hbs.get_prod_list_by_corp_code(80535346)
    print(data)


def get_simu_nav_by_code():
    hbs.set_token("qwertyuisdfghjkxcvbn1000")
    data, total_count = hbs.get_simu_nav_by_code("P0010", "20190101", "20191231")
    print(data)
    print(total_count)


if __name__ == "__main__":
    get_simu_nav_by_code()


hb_ip = '192.168.223.152'

sql_user_hb = {
    'ip': hb_ip,
    'user': 'readonly',
    'pass': 'c24mg2e6',
    'port': '3306'
}

sql_write_path_hb = {
    'commodities':
        'mysql+mysqldb://%s:%s@%s:%s/commodities?charset=utf8'
        % (sql_user_hb['user'], sql_user_hb['pass'], hb_ip, sql_user_hb['port']),
    'stocks':
        'mysql+mysqldb://%s:%s@%s:%s/stocks?charset=utf8'
        % (sql_user_hb['user'], sql_user_hb['pass'], hb_ip, sql_user_hb['port']),
    'work':
        'mysql+mysqldb://%s:%s@%s:%s/work?charset=utf8'
        % (sql_user_hb['user'], sql_user_hb['pass'], hb_ip, sql_user_hb['port']),
    'daily':
        'mysql+mysqldb://%s:%s@%s:%s/daily_data?charset=utf8'
        % (sql_user_hb['user'], sql_user_hb['pass'], hb_ip, sql_user_hb['port']),
}



#!/usr/bin/python
#coding:utf-8

"""
@author: Meng.lv
@contact: meng.lv@howbuy.com
@software: PyCharm
@file: quant_test.py.py
@time: 2020/10/20 16:13
"""
from datetime import datetime, timedelta
import hbshare as hbs

def main():

    hbs.set_token("eTFrL1JzRVI3WGJycEp2cXFCczhwZz09")

    end_date = datetime(2020, 10, 9).date()
    start_date = end_date - timedelta(days=365 * 2 + 7)
    start_date2 = end_date - timedelta(days=365 + 7)

    tab = hbs.gen_tab(end_date=end_date, class0='cta')
    #tab.render('summary.html')
    html_content = tab.render_embed()
    print("%html {}".format(html_content))


if __name__ == "__main__":
    main()


from datetime import datetime
import pandas as pd
import numpy as np
from sqlalchemy import create_engine
import pymysql

pymysql.install_as_MySQLdb()


def load_calendar(
        start_date=datetime(2010, 1, 1).date(),
        end_date=datetime.now().date(),
        freq='',
        db_path='',
        table='stocks_index'
):
    if freq.lower() in ['w', 'week', 'weekly']:
        freq_end = '_weekly'
    elif freq.lower() in ['month', 'monthly']:
        freq_end = '_monthly'
    else:
        freq_end = ''
    engine_stocks = create_engine(db_path)
    trade_cal = pd.read_sql_query(
        'select distinct t_date from ' + table + freq_end
        + '_ts where t_date>=' + start_date.strftime('%Y%m%d') + ' and t_date<='
        + end_date.strftime('%Y%m%d') + ' order by t_date',
        engine_stocks
    )
    return trade_cal


def load_calendar_extra(
        freq='',
        start_date=datetime(2010, 1, 1).date(),
        end_date=datetime.now().date(),
        db_path=''
):

    if freq.lower() in ['w', 'week', 'weekly']:
        cal_spe = load_calendar(start_date=start_date, end_date=end_date, freq=freq, db_path=db_path)
    elif freq.lower() in ['month', 'monthly']:
        cal_spe = load_calendar(start_date=start_date, end_date=end_date, freq=freq, db_path=db_path)
        cal_week = load_calendar(start_date=start_date, end_date=end_date, freq='w', db_path=db_path)
        cal_extra = cal_week[cal_week['t_date'] > cal_spe['t_date'][len(cal_spe) - 1]].reset_index(drop=True)
        if len(cal_extra) > 0:
            cal_spe = cal_spe.append(cal_extra.loc[len(cal_extra) - 1]).reset_index(drop=True)
    else:
        cal_spe = load_calendar(start_date=start_date, end_date=end_date, db_path=db_path)
    return cal_spe


# 输入产品列表，输出对应产品的周净值或月净值
# 输入：DataFrame，输出：DataFrame
# 如果first_date设置的太早会自动剔除前期空值
# 净值第一次变化的时间点为数据起点
def load_funds_data(
        fund_list, first_date=datetime.now().date(), end_date=datetime.now().date(),
        freq='w', fillna=True,
        db_path='', cal_db_path=''
):
    cal = load_calendar(end_date=end_date, db_path=cal_db_path)
    cal_weekly = load_calendar_extra(freq='w', end_date=end_date, db_path=cal_db_path)

    cal_spe = load_calendar_extra(freq=freq, end_date=end_date, db_path=cal_db_path)
    cal_all = cal_spe.copy()
    i_dates = []
    for i in range(len(fund_list)):
        fund_code = fund_list['code'][i].lower()
        fund_name = fund_list['name'][i]
        # print('Loading ' + fund_name + ' ' + freq + '；')

        fund_data = pd.read_sql_query(
            'select * from fund_data where code="' + fund_code
            + '" and t_date<=' + end_date.strftime('%Y%m%d')
            + ' order by t_date',
            create_engine(db_path)
        )[['t_date', 'nav']].rename(columns={'nav': fund_name})
        fund_data = fund_data.drop_duplicates(['t_date'], keep='first').reset_index(drop=True)
        fund_data = pd.merge(cal, fund_data, on='t_date', how='left')

        if fillna:
            latest_date_i = fund_data[fund_data[fund_name] > 0].index[-1]
            fund_data = fund_data.fillna(method='ffill')
            # 最新净值之后不填充空置
            if (
                    len(fund_data) > latest_date_i + 1
            ) and (
                    fund_data['t_date'][latest_date_i] <= cal_spe['t_date'].tolist()[-2]
            ):
                fund_data.loc[latest_date_i + 1:, fund_name] = np.nan

            # 算月度净值时先统一成周净值再转换成日净值，去掉部分日净值以同一所有产品净值频率，不直接按日净值转换为月净值
            if freq in ['month', 'monthly']:
                fund_data = pd.merge(cal_weekly, fund_data, on='t_date', how='left')
                nav_end_date = fund_data[fund_data[fund_name] > 0]['t_date'].tolist()[-1]
                fund_data = pd.merge(cal, fund_data, on='t_date', how='left')
                fund_data = fund_data.fillna(method='ffill')
                if cal_spe['t_date'].tolist()[-1] > nav_end_date:
                    date_i = cal_spe[cal_spe['t_date'] > nav_end_date]['t_date'].tolist()[0]
                    fund_data.loc[fund_data[fund_data['t_date'] >= date_i].index[0]:, fund_name] = np.nan
                else:
                    pass

        fund_data = pd.merge(cal_spe, fund_data, on='t_date', how='left')

        for d in range(len(fund_data)):
            if fund_data.iloc[d:d + 1][fund_name][d] == 1 and fund_data.iloc[d + 1:d + 2][fund_name][d + 1] != 1:
                fund_data = fund_data.iloc[d:].reset_index(drop=True)
                break
        i_dates.append(fund_data['t_date'][0])
        cal_all = pd.merge(cal_all, fund_data, on='t_date', how='left')

    i_date = min(i_dates)
    if i_date > first_date:
        first_date = i_date
    return cal_all[cal_all['t_date'] >= first_date].reset_index(drop=True)


# 输入产品列表，输出对应产品的周超额净值或月超额净值
# 输入：DataFrame，输出：DataFrame
# 输入的DataFrame中需要包含对应benchmark的code列
# 如果first_date设置的太早会自动剔除前期空值
# 指增产品只对净值大于1的序列进行计算，防止净值从1开始跳跃过大
def load_funds_outperformance(
        fund_list, first_date=datetime.now().date(), end_date=datetime.now().date(), freq='w',
        db_path='', cal_db_path='', index_table='stocks_index_ts'
):
    cal = load_calendar(end_date=end_date, db_path=cal_db_path)
    cal_weekly = load_calendar_extra(freq='w', end_date=end_date, db_path=cal_db_path)

    cal_spe = load_calendar_extra(freq=freq, end_date=end_date, db_path=cal_db_path)
    cal_eav = cal_spe.copy()
    cal_excess = cal_spe.copy()

    for i in range(len(fund_list)):
        fund_code = fund_list['code'][i].lower()
        fund_name = fund_list['name'][i]
        benchmark = fund_list['benchmark'][i]
        # print('Loading ' + fund_name + ' ' + freq + '；')
        # if first_date > fund_list['first_date'][i]:
        #     first_date = fund_list['first_date'][i]

        fund_data = pd.read_sql_query(
            'select t_date, nav from fund_data where code="' + fund_code
            + '" and t_date<=' + end_date.strftime('%Y%m%d')
            + ' order by t_date',
            create_engine(db_path)
        ).rename(columns={'nav': fund_name})
        fund_data = fund_data.drop_duplicates(['t_date'], keep='first').reset_index(drop=True)

        benchmark_data = pd.read_sql_query(
            'select t_date, close from ' + index_table
            + ' where t_date<=' + end_date.strftime('%Y%m%d')
            + ' and code="' + benchmark + '" order by t_date',
            create_engine(cal_db_path)
        ).rename(columns={'close': 'bm'})

        # fund_data = pd.merge(fund_data, benchmark_data, on='t_date', how='left')

        fund_aligned = pd.merge(cal, fund_data, on='t_date', how='left')
        latest_date_i = fund_aligned[fund_aligned[fund_name] > 0].index[-1]
        fund_aligned = fund_aligned.fillna(method='ffill')

        # 最新净值之后不填充空置
        if len(fund_aligned) > latest_date_i + 1:
            fund_aligned.loc[latest_date_i + 1:, fund_name] = np.nan

        # 算月度净值时先统一成周净值再转换成日净值，去掉部分日净值以同一所有产品净值频率，不直接按日净值转换为月净值
        if freq in ['month', 'monthly']:
            fund_aligned = pd.merge(cal_weekly, fund_aligned, on='t_date', how='left')
            fund_aligned = pd.merge(cal, fund_aligned, on='t_date', how='left')
            fund_aligned = fund_aligned.fillna(method='ffill')

        # fund_aligned = fund_aligned[fund_aligned['t_date'] >= datetime(2017, 7, 7).date()].reset_index(drop=True)
        fund_aligned = pd.merge(cal_spe['t_date'], fund_aligned, on='t_date', how='left')

        for d in range(len(fund_aligned)):
            if fund_aligned.iloc[d:d + 1][fund_name][d] == 1 and fund_aligned.iloc[d + 1:d + 2][fund_name][d + 1] != 1:
                fund_aligned = fund_aligned.iloc[d + 1:].reset_index(drop=True)
                break

        fund_aligned = pd.merge(fund_aligned, benchmark_data, on='t_date', how='left')
        fund_aligned['ret'] = fund_aligned[fund_name] / fund_aligned[fund_name].shift(1) - 1
        fund_aligned['bm_ret'] = fund_aligned['bm'] / fund_aligned['bm'].shift(1) - 1
        fund_aligned['excess'] = fund_aligned['ret'] - fund_aligned['bm_ret']
        fund_aligned['eav'] = (fund_aligned['excess'] + 1).cumprod()
        fund_aligned.loc[fund_aligned[fund_aligned['eav'] > 0].index[0] - 1, 'eav'] = 1
        cal_spe = pd.merge(cal_spe, fund_aligned[['t_date', fund_name]], on='t_date', how='left')
        cal_eav = pd.merge(
            cal_eav, fund_aligned[['t_date', 'eav']].rename(columns={'eav': fund_name}), on='t_date', how='left'
        )
        cal_excess = pd.merge(
            cal_excess,
            fund_aligned[['t_date', 'excess']].rename(columns={'excess': fund_name}),
            on='t_date',
            how='left'
        )

    i_date = fund_list['first_date'].min()
    if i_date > first_date:
        first_date = i_date

    result = {
        'nav': cal_spe[cal_spe['t_date'] >= first_date].reset_index(drop=True),
        'eav': cal_eav[cal_eav['t_date'] >= first_date].reset_index(drop=True),
        'excess': cal_excess[cal_excess['t_date'] >= first_date].reset_index(drop=True)
    }
    return result

